<?php

class Main extends Controller {
	
	private $userData = null;
	private $language = null;
	
	//список состояний приложения и соответствующих картинок
	private $states = array(
		'NEW'=>'state_01.png'
		, 'START'=>'state_02.png'
		, 'HALF'=>'state_03.png'
		, 'DONE'=>'state_04.png'
		, 'CANCELED'=>'state_05.png'
		, 'DELEGATED'=>'state_06.png'
		, 'IMPORTANT'=>'state_07.png'
		, 'ATTENTION'=>'state_08.png'
	);
    
	/**
	 * Конструктор
	 */
    function __construct() {
        parent::Controller();
//		$this->output->enable_profiler(TRUE);
    }

	/**
	 * Этот метод вызывается до любого другого метода контроллера.
	 * В нем проверяется прошел пользователь авторизации или нет.
	 * И соответственно ограничивается доступ к приложению.
	 * @param object $method имя вызванного метода
	 * @return 
	 */
    function _remap($method) {
		$this->load->library('user_agent');
        //для не авторизовавшегося пользователя доступ закрыт ко всем методам
        //кроме login и checklogin
		$allowedPages = array('login', 'checklogin', 'register', 'checkregister');
		if (($this->userData = $this->redux_auth->profile()) !== FALSE) {
			//получаем язык из БД и загружаем соответствующий файлы переводов
			$this->load->model('moptions');

			$language = $this->moptions->getOption($this->userData->id, 'language');
			if ($language !== FALSE) {
				$this->language = $language[0]->o_value;
				$this->lang->load('todo', $this->language);
			}
			else if ($this->agent->accept_lang('ru')) {
				$this->lang->load('todo', 'russian');
				$this->language = 'russian';
			}
			else {
				$this->lang->load('todo', 'english');
				$this->language = 'english';
			}
            $this->$method();
		}
        else if (in_array($method, $allowedPages)) {
        	//определяем язык по заголовку Accept-encoding
			//и загружаем соответствующий файл переводов

			if ($this->agent->accept_lang('ru')) {
				$this->lang->load('todo', 'russian');
				$this->language = 'russian';
			}
			else {
				$this->lang->load('todo', 'english');
				$this->language = 'english';
			}
            $this->$method();
        }
        else {
        	//если посетитель не авторизовался и хочет зайти на
			//закрытую страницу - редирект на страницу входа
            redirect('main/login');
        }
    }
    
    /**
     * Создает страницу с формой входа
     */
    function login() {
    	//если вход выполнен - редирект на страницу со списком записей
        if ($this->redux_auth->logged_in()) {
            redirect('main/index');
        }
		
		//формируем страницу
        $pageData['title'] = lang('title_login');
        
        $this->load->helper('form');
        
        $this->load->view('header', $pageData);
        $this->load->view('messageblock');
        $this->load->view('loginform');
        $this->load->view('footer', array('curLanguage'=>$this->language));
    }
    
    /**
     * Проверяет имя пользователя и пароль. Если соответствующий пользователь
     * найден - выполняется переход на главную страницу приложения
     * (main/index), в противном случае - выводим сообщения об ошибке.
     */
    function checklogin() {
        $this->load->helper('form');
		//настраиваем библиотеку form_validation
        $this->load->library('form_validation');
		$this->form_validation->set_error_delimiters('<p class="error">', '</p>');
        $this->form_validation->set_rules('u_name'
			, 'lang:login_u_name', 'required');
        $this->form_validation->set_rules('u_pass'
			, 'lang:login_u_pass', 'required');
		//проверяем полученные данные
        if ($this->form_validation->run() === FALSE) {
            //форма заполнена не правильно (не введены имя или пароль)
            $pageData['title'] = lang('title_login_err');
            
            $this->load->view('header', $pageData);
            $this->load->view('messageblock');
            $this->load->view('loginform');
            $this->load->view('footer', array('curLanguage'=>$this->language));
        }
        else {
            //форма заполнена правильно
            $login    = $this->input->post('u_name');
            $password = $this->input->post('u_pass');
            
			//проверяем соответствие введенного имени и пароля
            if ($this->redux_auth->login($login, $password)) {
                //пользователь найден
                redirect('main/index');
            }
            else {
                //пользователь не найден
                $this->session->set_flashdata('message', lang('login_err'));
                redirect('main/login');
            }
        }
    }
    
    /**
     * Выход из приложения и редирект на страницу main/login.
     */
    function logout() {
        $this->redux_auth->logout();
        redirect('main/login');
    }
    
    /**
     * Главная страница приложения.
     */
    function index() {
        $this->load->model('mtodonotes');
        
        $pageData['title'] = lang('title_main');
		//получаем список записей
        $pageData['notes'] = $this->mtodonotes->getAllNotes($this->userData->id);
        
        $this->load->helper('form');
        
		//формируем страницу
        $this->load->view('header', $pageData);
        $this->load->view('mainmenu');
        $this->load->view('messageblock');
        $this->load->view('todolist');
        $this->load->view('footer', array('curLanguage'=>$this->language));
    }
    
	/**
	 * На этой странице будет показан список не завершенных задач, т.е.
	 * с состоянием NEW, START, HALF, IMPORTANT или ATTENTION.
	 * @return 
	 */
	function uncomplete() {
		$this->load->model('mtodonotes');
		
        $pageData['title'] = lang('title_main');
		//получаем список записей
        $pageData['notes'] = $this->mtodonotes->getUncomplete($this->userData->id);
        
        $this->load->helper('form');
        
		//формируем страницу
        $this->load->view('header', $pageData);
        $this->load->view('mainmenu');
        $this->load->view('messageblock');
        $this->load->view('todolist');
        $this->load->view('footer', array('curLanguage'=>$this->language));
	}
	
	/**
	 * На этой странице будет показан список важных задач, т.е.
	 * с состояниями IMPORTANT или ATTENTION.
	 * @return 
	 */
	function important() {
		$this->load->model('mtodonotes');
		
        $pageData['title'] = lang('title_main');
		//получаем список записей
        $pageData['notes'] = $this->mtodonotes->getImportant($this->userData->id);
        
        $this->load->helper('form');
        
		//формируем страницу
        $this->load->view('header', $pageData);
        $this->load->view('mainmenu');
        $this->load->view('messageblock');
        $this->load->view('todolist');
        $this->load->view('footer', array('curLanguage'=>$this->language));
	}
	
	/**
	 * На этой странице будет показан список задач с сегодняшней датой.
	 * @return 
	 */
	function fortoday() {
		$this->load->model('mtodonotes');
		
        $pageData['title'] = lang('title_main');
		//получаем список записей
        $pageData['notes'] = $this->mtodonotes->getForToday($this->userData->id);
        
        $this->load->helper('form');
        
		//формируем страницу
        $this->load->view('header', $pageData);
        $this->load->view('mainmenu');
        $this->load->view('messageblock');
        $this->load->view('todolist');
        $this->load->view('footer', array('curLanguage'=>$this->language));
	}
	
	/**
	 * На этой странице размещена форма, позволяющая фильтровать задачи.
	 * @return 
	 */
    function filters() {
        $pageData['title'] = lang('title_filters');
		$pageData['searchDone'] = FALSE;
		$pageData['hasResults'] = FALSE;

	    $this->load->helper('form');
		//настраиваем form_validation
	    $this->load->library('form_validation');
		//эти правила нужно установить для того,
		//чтобы работало восстановление значений для чекбоксов
	    $this->form_validation->set_rules('state[]', 'NEW');
	    $this->form_validation->set_rules('state[]', 'START');
	    $this->form_validation->set_rules('state[]', 'HALF');
	    $this->form_validation->set_rules('state[]', 'DONE');
	    $this->form_validation->set_rules('state[]', 'CANCELED');
	    $this->form_validation->set_rules('state[]', 'DELEGATED');
	    $this->form_validation->set_rules('state[]', 'IMPORTANT');
	    $this->form_validation->set_rules('state[]', 'ATTENTION');
	    $this->form_validation->set_rules('from_date'
			, 'lang:from_date', 'required|callback_date_check');
	    $this->form_validation->set_rules('to_date', 'lang:to_date'
			, 'required|callback_date_check');
		//проверяем полученные данные
	    if ($this->form_validation->run() === FALSE) {
	        //ошибка
			//описания ошибок будут выведены около соответствующих полей
	    } else {
	        //форма заполнена правильно
			//формируем массив с данными для поиска
			$searchData['states'] = $this->input->post('state');
			$searchData['from'] = $this->input->post('from_date');
			$searchData['to'] = $this->input->post('to_date');
			
			$this->load->model('mtodonotes');
			//запускаем поиск
	        $pageData['notes'] = $this->mtodonotes->find($this->userData->id
				, $searchData);
			//в зависимости от результатов устанавливаем hasResults и searchDone
			if ($pageData['notes'] !== FALSE) {
				$pageData['hasResults'] = TRUE;
			}
			if ($this->input->post('filterSbm')) {
				$pageData['searchDone'] = TRUE;
			}
	    }

		//формируем страницу
        $this->load->view('header', $pageData);
        $this->load->view('mainmenu');
        $this->load->view('messageblock');
		$this->load->view('filter.php');
		//показываем результаты поиска если они есть
		if ($pageData['hasResults']) {
	        $this->load->view('todolist');
		}
        $this->load->view('footer', array('curLanguage'=>$this->language));
	}
	
	/**
	 * AJAX метод. Изменяет состояние записи.
	 * Параметры (передаются в массиве $_POST):
	 * n_id - id записи, которую нужно изменить
	 * n_state - новое состояние (передается ключ массива $states)
	 * @return 
	 */
    function updatestate() {
	    $res = array();
	       
		//настраиваем form_validation
	    $this->load->library('form_validation');
	        
	    $this->form_validation->set_rules('n_id', 'lang:n_id', 'required|integer');
	    $this->form_validation->set_rules('n_state', 'lang:n_state', 'required');
	        
	    if ($this->form_validation->run() === FALSE) {
	        //ошибка
	        $res['status'] = 'ERR';
	        $res['message'] = validation_errors();
	    } else {
	        //форма заполнена правильно
	        $id = $this->input->post('n_id');
			$noteData['td_state'] = $this->input->post('n_state');
	        //сохраняем запись
	        $this->load->model('mtodonotes');
	        if ($this->mtodonotes->updateNote($this->userData->id, $id, $noteData)) {
	            $res['status'] = 'OK';
	            $res['id'] = $id;
				$res['new_state'] = $this->states[$noteData['td_state']];
	        } else {
	            $res['status'] = 'ERR';
	            $res['message'] = lang('update_db_err');
	        }
	    }
	    //возвращаем результат    
	    echo json_encode($res);
	}

	/**
	 * AJAX метод. Изменяет текст записи.
	 * Параметры (передаются в массиве $_POST):
	 * id - id блока на странице в котором находится запись, которую нужно изменить
	 * (id блоков не могут начинаться с цифры, поэтому чтобы получить id записи
	 * нужно из id блока вырезать приставку (note_text_))
	 * value - новый текст
	 * @return 
	 */
    function updatetext() {
    	//получаем параметры
	    $id = $this->input->post('id');
	    $text = $this->input->post('value');
	    
		//получаем id записи
	    if ($id !== FALSE) {
	        $id = substr($id, 10);
	    } else {
	        echo lang('update_id_err');
	        return;
	    }
	        
	    if ($text === FALSE || $text == '') {
	        echo lang('update_text_err');
	        return;
	    }
	    
		//изменяем запись в БД
	    $this->load->model('mtodonotes');
	    if ($this->mtodonotes->updateNote($this->userData->id, $id, array('td_text'=>$text))) {
	    	//возвращаем измененный текст
	        echo $text;
	    } else {
	    	//или сообщение об ошибке
	        echo lang('update_db_err');
	    }
	}

	/**
	 * AJAX метод. Изменяет дату записи.
	 * Параметры (передаются в массиве $_POST):
	 * id - id блока на странице в котором находится запись, которую нужно изменить
	 * (id блоков не могут начинаться с цифры, поэтому чтобы получить id записи
	 * нужно из id блока вырезать приставку (note_date_))
	 * value - новая дата
	 * @return 
	 */
	function updatedate() {
	    $id = $this->input->post('id');
	    $date = $this->input->post('value');
	        
	    if ($id !== FALSE) {
	        $id = substr($id, 10);
	    } else {
	        echo lang('update_id_err');
	        return;
	    }
	        
	    if ($date === FALSE || preg_match('/^\d{4}-\d{2}-\d{2}$/', $date) != 1) {
	        echo lang('update_date_err');
	        return;
	    }
	        
		//сохраняем новую дату в базе
	    $this->load->model('mtodonotes');
	    if ($this->mtodonotes->updateNote($this->userData->id, $id, array('td_date'=>$date))) {
	    	//и возвращаем измененную дату
	        echo $date;
	    } else {
	    	//или сообщение об ошибке
	        echo lang('update_db_err');
	    }
	}
	
	/**
	 * AJAX метод. Изменяет все параметры записи.
	 * Параметры (передаются в массиве $_POST):
	 * id - id блока на странице в котором находится запись, которую нужно изменить
	 * (id блоков не могут начинаться с цифры, поэтому чтобы получить id записи
	 * нужно из id блока вырезать приставку (note_date_))
	 * value - новая дата
	 * @return 
	 */
	function updatenote() {
		//настраиваем form_validation
	    $this->load->library('form_validation');
	        
	    $this->form_validation->set_rules('n_id', 'lang:n_id', 'required|integer');
	    $this->form_validation->set_rules('n_state', 'lang:n_state', 'required');
	    $this->form_validation->set_rules('n_date'
			, 'lang:n_date', 'required|callback_date_check');
	    $this->form_validation->set_rules('n_text'
			, 'lang:n_text', 'required|min_length[1]|max_length[4000]');
	    
	    if ($this->form_validation->run() === FALSE) {
	        //ошибка
	        $res['status'] = 'ERR';
	        $res['message'] = validation_errors();
	    } else {
	        //форма заполнена правильно
		    $id = $this->input->post('n_id');
		    $noteData['td_date'] = $this->input->post('n_date');
		    $noteData['td_state'] = $this->input->post('n_state');
		    $noteData['td_text'] = $this->input->post('n_text');
	    	
	        //сохраняем запись
	        $this->load->model('mtodonotes');
	        if ($this->mtodonotes->updateNote($this->userData->id, $id, $noteData)) {
	            $res['status'] = 'OK';
	            $res['id'] = $id;
				$res['new_img'] = base_url().'images/'
					.$this->states[$noteData['td_state']];
				$res['new_state'] = $noteData['td_state'];
				$res['new_text'] = $noteData['td_text'];
				$res['new_date'] = $noteData['td_date'];
	        } else {
	            $res['status'] = 'ERR';
	            $res['message'] = lang('update_db_err');
	        }
	    }
	    //возвращаем результат    
	    echo json_encode($res);
	}
    
    /**
	 * AJAX метод. Сохраняет новую запись в базе.
	 * Параметры (передаются в массиве $_POST):
	 * n_date - дата новой записи
	 * n_text - текст записи
     */
    function addnote() {
        
        $res = array();
        
		//настраиваем библиотеку form_validation
        $this->load->library('form_validation');
        $this->form_validation->set_rules('n_text', 'lang:n_text', 'required');
        $this->form_validation->set_rules('n_date'
			, 'lang:n_date', 'required|exact_length[10]|callback_date_check');
        if ($this->form_validation->run() === FALSE) {
            //ошибка
            $res['status'] = 'ERR';
            $res['message'] = validation_errors();
        }
        else {
            //форма заполнена правильно
            //получаем введенные данные...
            $noteData['td_text'] = $this->input->post('n_text');
            $noteData['td_date'] = $this->input->post('n_date');
                
            //...и сохраняем запись
            $this->load->model('mtodonotes');
            $noteData['td_state'] = 'NEW'; // состояние новой записи всегда NEW
            if (($id = $this->mtodonotes->addNote($this->userData->id, $noteData)) !== FALSE) {
                $res['status'] = 'OK';
				$res['id'] = $id;
				$res['state_img_url'] = base_url().'images/state_01.png';
				$res['state'] = $noteData['td_state'];
				$res['text'] = $noteData['td_text'];
				$res['date'] = $noteData['td_date'];
            }
            else {
                $res['status'] = 'ERR';
                $res['message'] = lang('add_db_err');
            }
        }
        //возвращаем результат
        echo json_encode($res);
    }
	
	/**
	 * AJAX метод. Изменяет удаляет запись.
	 * Параметры (передаются в массиве $_POST):
	 * n_id - id записи, которую нужно удалить
	 * @return JSON строка с описанием результа операции или ошибки
	 */
	function removenote() {
		$res = array();
		
		//настраиваем form_validation
		$this->load->library('form_validation');
		
		$this->form_validation->set_rules('n_id', 'lang:n_id', 'required|integer');
		
        if ($this->form_validation->run() === FALSE) {
            $res['status'] = 'ERR';
            $res['message'] = validation_errors();
        }
        else {
        	$id = $this->input->post('n_id');
        	//удаляем запись
			$this->load->model('mtodonotes');
			if ($this->mtodonotes->removenote($this->userData->id, $id)) {
				$res['status'] = 'OK';
				$res['id'] = $id;
			}
			else {
				$res['status'] = 'ERR';
				$res['message'] = lang('remove_db_err');
			}
		}
		//возвращаем результат
		echo json_encode($res);
	}
	
	/**
	 * Этот метод выполняет проверку формата даты
	 * @param object $str строка с датой
	 * @return TRUE - если дата записана правильно, FALSE - в противном случае
	 */
	function date_check($str) {
		if (preg_match('/^\d{4}-\d{2}-\d{2}$/', $str) == 1) {
			return TRUE;
		}
		else {
			$this->form_validation->set_message('date_check', lang('add_date_err'));
			return FALSE;
		}
	}
	
	/**
	 * Этот метод показывает страницу с формой регистрации
	 * @return 
	 */
	function register() {
    	//если вход выполнен - редирект на страницу со списком записей
        if ($this->redux_auth->logged_in()) {
            redirect('main/index');
        }

		//формируем страницу
        $pageData['title'] = lang('title_register');
        
        $this->load->helper('form');
        
        $this->load->view('header', $pageData);
        $this->load->view('messageblock');
        $this->load->view('registerform');
        $this->load->view('footer', array('curLanguage'=>$this->language));
	}
	
	/**
	 * Этот метод регистрирует нового пользователя
	 * @return 
	 */
	function checkregister() {
        
		//формируем страницу
        $pageData['title'] = lang('title_register');

		$this->load->helper('form');
		//настраиваем библиотеку form_validation
        $this->load->library('form_validation');
		$this->form_validation->set_error_delimiters('<p class="error">', '</p>');
		$this->form_validation->set_rules('r_name'
			, 'lang:register_name', 'required');
		$this->form_validation->set_rules('r_email'
			, 'lang:register_email', 'required|valid_email');
		$this->form_validation->set_rules('r_pass'
			, 'lang:register_pass', 'required|min_length[5]');
		$this->form_validation->set_rules('r_pass_conf'
			, 'lang:register_pass_conf', 'required|matches[r_pass]');
		if ($this->form_validation->run() === FALSE) {
            //форма заполнена не правильно
            $pageData['title'] = lang('title_register_err');
            
            $this->load->view('header', $pageData);
            $this->load->view('messageblock');
            $this->load->view('registerform');
            $this->load->view('footer', array('curLanguage'=>$this->language));
		}
		else {
            //форма заполнена правильно
            $name     = $this->input->post('r_name');
            $email    = $this->input->post('r_email');
            $password = $this->input->post('r_pass');
            
			//регистрируем пользователя
            if ($this->redux_auth->register($name, $password, $email)) {
                $this->session->set_flashdata('message', lang('register_ok'));
				redirect('main/login');
            }
            else {
                //пользователь не найден
                $this->session->set_flashdata('message', lang('register_err'));
                redirect('main/login');
            }
		}
	}
	
	/**
	 * Этот метод создает страницу с помощью которой можно изменить
	 * настройки пользователя
	 * @return 
	 */
	function options() {
        $pageData['title'] = lang('title_options');
		$this->load->helper('form');

		//Показываем страницу с формой
        $this->load->view('header', $pageData);
        $this->load->view('mainmenu');
        $this->load->view('messageblock');
        $this->load->view('optionsform', array('curLanguage'=>$this->language));
        $this->load->view('footer', array('curLanguage'=>$this->language));
	}
	
	function changepass() {
        $pageData['title'] = lang('title_options');
		$this->load->helper('form');

		if ($this->input->post('o_set_pass') !== false) {
			$this->load->library('form_validation');
			$this->form_validation->set_rules('o_old_pass'
				, 'lang:options_old_pass', 'required');
			$this->form_validation->set_rules('o_pass'
				, 'lang:options_new_pass', 'required|min_length[5]');
			$this->form_validation->set_rules('o_pass_conf'
				, 'lang:options_new_pass_conf', 'required|matches[o_pass]');
			if ($this->form_validation->run() === FALSE) {
				//ошибка
				$pageData['result'] = lang('options_canceled');
			}
			else {
				//форма заполнена правильно
		        $old = $this->input->post('o_old_pass');
		        $new = $this->input->post('o_pass');
		        
		        $identity = $this->session->userdata($this->config->item('identity'));
		        $change = $this->redux_auth->change_password($identity, $old, $new);
				
	    		if ($change) {
					$this->session->set_flashdata('message'
						, lang('options_new_pass_set'));
	    			$this->logout();
					return;
	    		}
	    		else {
	    			$pageData['result'] = lang('options_change_pass_err');
	    		}
			}
		}
		
		//Показываем страницу с формой
        $this->load->view('header', $pageData);
        $this->load->view('mainmenu');
        $this->load->view('messageblock');
        $this->load->view('optionsform', array('curLanguage'=>$this->language));
        $this->load->view('footer', array('curLanguage'=>$this->language));
	}
	
	function changelanguage() {
        $pageData['title'] = lang('title_options');
		$this->load->helper('form');

		if ($this->input->post('o_set_lang') !== false) {
			$this->load->library('form_validation');
			$this->form_validation->set_rules('o_lang'
				, 'lang:options_cur_lang', 'required');
			if ($this->form_validation->run() === FALSE) {
				//ошибка
				$pageData['result'] = lang('options_canceled');
			}
			else {
				//форма заполнена правильно
		        $language = $this->input->post('o_lang');
		        
				//если опция language существует, изменяем ее...
				if ($this->moptions->getOption($this->userData->id, 'language')) {
					$this->moptions->updateOption($this->userData->id, 'language', $language);
				}
				//...если нет - устанавливаем новое значение
				else {
					$this->moptions->addOption($this->userData->id, 'language', $language);
				}

				redirect('main/options');
				return;
			}
		}
		
		//Показываем страницу с формой
        $this->load->view('header', $pageData);
        $this->load->view('mainmenu');
        $this->load->view('messageblock');
        $this->load->view('optionsform', array('curLanguage'=>$this->language));
        $this->load->view('footer', array('curLanguage'=>$this->language));
	}
}

// end of main.php