<?php
/**
 * Этот класс предназначен для работы с таблицей опций, в
 * которой сохраняются настройки пользователей.
 * Все настройки сохраняются в формате
 * ключ = значение
 */
class MOptions extends Model {
	
    public function __construct() {
        parent::Model();
    }
	
	/**
	 * Создает новую опцию для текущего пользователя
	 * @param object $userId id текущего пользователя в БД
	 * @param object $key ключ
	 * @param object $value значение
	 * @return id созданной опции или FALSE при возникновении ошибки
	 */
	public function addOption($userId, $key, $value) {
		$data = array('o_user_id'=>$userId
						,'o_key'=>$key
						,'o_value'=>$value);
		if ($this->db->insert('options', $data)) {
			return $this->db->insert_id();
		}
		return FALSE;
	}
	
	/**
	 * Читает заданную опцию
	 * @param object $userId id текущего пользователя в БД
	 * @param object $key ключ
	 * @return объект с данными опции или FALSE при возникновении ошибки
	 */
	public function getOption($userId, $key) {
		$this->db->where('o_user_id', $userId);
		$this->db->where('o_key', $key);
		$query = $this->db->get('options');
		if ($query->num_rows() > 0) {
			return $query->result();
		}
		else {
			return false;
		}
	}
	
	/**
	 * Удаляет указанную опцию
	 * @param object $userId id текущего пользователя в БД
	 * @param object $key ключ
	 * @return true в случае успешного удаления, false - при возникновении ошибки
	 */
	public function removeOption($userId, $key) {
		$this->db->where('o_user_id', $userId);
		$this->db->where('o_key', $key);
		return $this->db->delete('options');
	}
	
	/**
	 * Изменяет значение указанной опции
	 * @param object $userId id текущего пользователя в БД
	 * @param object $key ключ
	 * @param object $value новое значение
	 * @return true в случае успешного изменения, false - при возникновении ошибки
	 */
	public function updateOption($userId, $key, $value) {
		$data = array('o_value'=>$value);
		$this->db->where('o_user_id', $userId);
		$this->db->where('o_key', $key);
		return $this->db->update('options', $data);
	}
	
	/**
	 * Удаляет все опции текущего пользователя
	 * @param object $userId id текущего пользователя в БД
	 * @return true в случае успешного удаления, false - при возникновении ошибки
	 */
	public function removeAllUserOptions($userId) {
		$this->db->where('o_user_id', $userId);
		return $this->db->delete('options');
	}
}
//end of moptions.php