<?php

class MToDoNotes extends Model {
    
    function __construct() {
        parent::Model();
    }
    
    /**
     * Ищет все записи.
	 * @param object $userId id текущего пользователя в БД
     * @return объект, содержащий все найденные записи,
     * если записи отсутствуют - false.
     */
    function getAllNotes($userId) {
    	$this->db->where('td_user_id', $userId);
        $query = $this->db->get('todonotes');
        if ($query->num_rows() > 0) {
            return $query->result();
        }
        else {
            return false;
        }
    }

	/**
	 * Ищет все не завершенные задачи, т.е. с состоянием
	 * NEW, START, HALF, IMPORTANT или ATTENTION.
	 * @param object $userId id текущего пользователя в БД
	 * @return объект, содержащий все найденные записи,
     * если записи отсутствуют - false.
	 */
	function getUncomplete($userId) {
		$states = array('NEW', 'START', 'HALF', 'IMPORTANT', 'ATTENTION');
    	$this->db->where('td_user_id', $userId);
		$this->db->where_in('td_state', $states);
        $query = $this->db->get('todonotes');
        if ($query->num_rows() > 0) {
            return $query->result();
        }
        else {
            return false;
        }
	}
	
	/**
	 * Ищет все важные задачи, т.е. с состоянием
	 * IMPORTANT или ATTENTION.
	 * @param object $userId id текущего пользователя в БД
	 * @return объект, содержащий все найденные записи,
     * если записи отсутствуют - false.
	 */
	function getImportant($userId) {
		$states = array('IMPORTANT', 'ATTENTION');
    	$this->db->where('td_user_id', $userId);
		$this->db->where_in('td_state', $states);
        $query = $this->db->get('todonotes');
        if ($query->num_rows() > 0) {
            return $query->result();
        }
        else {
            return false;
        }
	}
	
	/**
	 * Ищет все задачи с сегодняшней датой.
	 * @param object $userId id текущего пользователя в БД
	 * @return объект, содержащий все найденные записи,
     * если записи отсутствуют - false.
	 */
	function getForToday($userId) {
    	$this->db->where('td_user_id', $userId);
		$this->db->where('td_date = CURDATE()');
        $query = $this->db->get('todonotes');
        if ($query->num_rows() > 0) {
            return $query->result();
        }
        else {
            return false;
        }
	}
    
	/**
	 * Сохраняет новую задачу в БД. 
	 * @param object $userId id текущего пользователя в БД
	 * @param object $noteData массив с данными записи, формируется контроллером.
	 * @return id новой записи (если она создана), false - в случае возникновения ошибки.
	 */
    function addNote($userId, $noteData) {
    	$noteData['td_user_id'] = $userId;
        if ($this->db->insert('todonotes', $noteData)) {
        	return $this->db->insert_id();
        }
		return FALSE;
    }
	
	/**
	 * Изменяет существующую запись в БД
	 * @param object $userId id текущего пользователя в БД
	 * @param object $id id записи, которую нужно изменить
	 * @param object $noteData массив с данными записи, формируется контроллером
	 * @return true - если операция успешно выполнена, false - в противном случае
	 */
	function updateNote($userId, $id, $noteData) {
    	$this->db->where('td_user_id', $userId);
		$this->db->where('id', $id);
		return $this->db->update('todonotes', $noteData);
	}
	
	/**
	 * Удаляет запись из БД
	 * @param object $userId id текущего пользователя в БД
	 * @param object $id id записи, которую нужно удалить
	 * @return true - если операция успешно выполнена, false - в противном случае
	 */
	function removeNote($userId, $id) {
    	$this->db->where('td_user_id', $userId);
		$this->db->where('id', $id);
		return $this->db->delete('todonotes');
	}
	
	/**
	 * Ищет записи в БД по заданным параметрам
	 * @param object $userId id текущего пользователя в БД
	 * @param object $searchData массив с параметрами, формируется контроллером на
	 * основе формы, которую заполнил пользователь
	 * @return объект, содержащий все найденные записи,
     * если записи не найдены - false.
	 */
	function find($userId, $searchData) {
		if ($searchData['states'] !== FALSE) {
			$this->db->where_in('td_state', $searchData['states']);
		}
    	$this->db->where('td_user_id', $userId);
		$this->db->where('td_date >=', $searchData['from']);
		$this->db->where('td_date <=', $searchData['to']);
		$query = $this->db->get('todonotes');
		if ($query->num_rows() > 0) {
			return $query->result();
		}
		else {
			return FALSE;
		}
	}
}

// end of mtodonotes.php